/*
 * The Kibitz HTTP daemon
 *
 * (c) 2005-2012 Tomas Valla <tom@ucw.cz>
 *
 * This software may be freely distributed and used according to the terms
 * of the GNU Lesser General Public License. See COPYING for details.
 */

#include <kibitz/lib.h>
#include <ucw/conf.h>
#include <ucw/getopt.h>
#include <ucw/stkstring.h>
#include <kibitz/kibitz.h>
#include <kibitz/internal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

static char *options = CF_SHORT_OPTS "c:i:p:v";

static void NONRET usage(char *msg)
{
	if (msg)
		fputs(msg, stderr);
	fprintf(stderr, "\nUsage: khttpd [-v] [-c cfgname] [-i idxname]\n"
		"  -v		log all network connections\n"
		"  -c cfgname	load config file cfgname, default is etc/kibitz\n"
		"  -i idxname	load index idxname, default is index.idx\n"
		"  -p port	the port number where to listen, default is in etc/kibitz\n");
	exit(1);
}

int main(int argc, char **argv)
{
	char idx[256] = "index.idx";
	char cfg[256] = "etc/kibitz";
	int opt;
	int verbose = 0;
	int port = 0;

	log_init(argv[0]);
	idx[sizeof(idx)-1] = cfg[sizeof(cfg)-1] = 0;

	while ((opt = cf_getopt(argc, argv, options, CF_NO_LONG_OPTS, NULL)) >= 0)
		switch (opt) {
		case 'c':
			strncpy(cfg, optarg, sizeof(cfg)-1);
			break;
		case 'i':
			strncpy(idx, optarg, sizeof(idx)-1);
			break;
		case 'p':
			{
				char *e;
				port = strtol(optarg, &e, 10);
				if (!*optarg || *e)
					usage("Invalid port number");
			}
		case 'v':
			verbose = 1;
			break;
		default:
			usage("Invalid option specified");
		}

	log_file(config.httpd_log_file);
	kibitz_init(cfg);
	if (verbose)
		cf_set("Kibitz.LogConnections=1");
	if (port)
		cf_set(stk_printf("Kibitz.Port=%d", port));

	kibitz_httpd_init(idx, cfg);
	kibitz_httpd_exec();
	kibitz_close_index();
	kibitz_cleanup();

	return 0;
}
