#!/bin/bash
#
#  Kibitz Search Server Keeper Script
#  (c) 2002--2003 Martin Mares <mj@ucw.cz>
#  (c) 2006 Vladimir Jelen <vladimir.jelen@netcentrum.cz>
#

ulimit -c unlimited

if [ "$UID" = 0 ] ; then
	echo >&2 "kibitz-keeper: Must not be run as root!"
	exit 1
fi
if [ "$1" == start -o "$1" == stop -o "$1" == restart ] ; then
	echo >&2 "kibitz-keeper: You probably meant kibitz-control $@, didn't you?"
	exit 1
fi

eval `bin/config HTTPD Port=3333 StatusFile '*'`
eval `bin/config KibitzKeeper TestRetry TestWait CrashMail RotateLogs CrashWaitThreshold CrashWaitCeiling DaemonPIDFile KeeperPIDFile TestQuery SwapLock SwapLockTimeout`

DHO="--pid-file=$CF_DaemonPIDFile"
if [ -n "$CF_StatusFile" ] ; then
	DHO="$DHO --status-file=$CF_StatusFile"
fi

exec </dev/null >/dev/null 2>&1

function send_report
{
	if [ -n "$CF_CrashMail" ] ; then
		(
			HOST=`hostname`
			HOSTF=`hostname -f`
			CRM=`echo "$CF_CrashMail" | sed 's/ /, /g'`
			cat <<EOF
From: "Kibitz at $HOST" <`whoami`>
To: $CRM
Reply-To: $CRM
Subject: khttpd $@

With sorrow in our hearts, we announce that khttpd $@.

Host:			$HOSTF
Current directory:	`pwd`
Date and time:		`date`
Last words:
EOF
			tail -10 `ls -1t log/khttpd* | head -1`
		) | /usr/sbin/sendmail $CF_CrashMail
	fi
}

leow=0
while true ; do
	bin/daemon-helper --start $DHO -- `pwd`/bin/khttpd "$@"
	RC=$?
	[ $RC = 101 ] && exit 0  # Already running
	if [ -n "$CF_StatusFile" -a ! -s "$CF_StatusFile" ] ; then
		send_report "failed to start"
		exit 1
	else
		send_report "exited with return code $RC"
	fi
	now=`date +%s`
	if [ $(($now-$leow)) -gt $CF_CrashWaitThreshold ] ; then
		delay=1
	elif [ $delay -lt $CF_CrashWaitCeiling ] ; then
		delay=$((2*$delay))
	fi
	sleep $delay
	leow=$now
done
