/*
 * The Kibitz Indexer
 *
 * (c) 2005-2012 Tomas Valla <tom@ucw.cz>
 *
 * This software may be freely distributed and used according to the terms
 * of the GNU Lesser General Public License. See COPYING for details.
 */

#include <kibitz/lib.h>
#include <ucw/conf.h>
#include <ucw/fastbuf.h>
#include <ucw/getopt.h>
#include <kibitz/kibitz.h>
#include <custom/custom.h>
#include <kibitz/internal.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>

static char *options = CF_SHORT_OPTS "c:f:hi:sv";

static void NONRET usage(char *msg)
{
	if (msg)
		fputs(msg, stderr);
	fprintf(stderr, "\nUsage: kindexer [-v] [-c cfgname] [-s] [-f inputname] [-i idxname]\n"
	    "   -v		make indexer verbose\n"
	    "   -h		this help text\n"
	    "   -c cfgname	load config file cfgname, default is etc/kibitz\n"
	    "   -s		read standard input as data source\n"
	    "   -f inputname	read file inputname as data source\n"
	    "   -i idxname	create index idxname, default is index.idx\n");
	exit(1);
}

int main(int argc, char **argv)
{
	char idx[256] = "index.idx";
	char cfg[256] = "etc/kibitz";
	int opt;
	int verbose = 0;
	struct fastbuf *infb = NULL;

	log_init(argv[0]);
	idx[sizeof(idx)-1] = cfg[sizeof(cfg)-1] = 0;

	while ((opt = cf_getopt(argc, argv, options, CF_NO_LONG_OPTS, NULL)) >= 0)
		switch (opt) {
		case 'c':
			strncpy(cfg, optarg, sizeof(cfg)-1);
			break;
		case 'f':
			if (infb)
				usage("Input already specified");
			infb = bopen(optarg, O_RDONLY, 65536);
			break;
		case 'h':
			usage(NULL);
		case 'i':
			strncpy(idx, optarg, sizeof(idx)-1);
			break;
		case 's':
			if (infb)
				usage("Input already specified");
			infb = bfdopen(0, 65536);
			break;
		case 'v':
			verbose = 1;
			break;
		default:
			usage("Invalid option specified");
		}

	log_file(config.indexer_log_file);
	kibitz_init(cfg);
	if (verbose)
		cf_set("Kibitz.Verbose=1");
	if (!infb)
		usage("No input specified");
	struct fastbuf *idxfb = bopen(idx, O_WRONLY | O_CREAT | O_TRUNC, 65536);

	msg(L_INFO, "Indexing input into index '%s'", idx);
	uns n = kibitz_make_index(infb, idxfb);
	bclose(infb);
	bclose(idxfb);
	msg(L_INFO, "Indexed %d phrases", n);

	kibitz_cleanup();
	return 0;
}
